{$A+,B-,D-,E-,F-,I+,L-,N-,O-,R-,S+,V+}
UNIT extStrings;

INTERFACE

CONST
  TAB = #9;
  LF  = #10;
  CR  = #13;
  ESC = #27;
  SPC = #32;

TYPE
  CharSet = SET OF Char; { in Pascal kann SET of Char nicht direkt in
                           Funktionen/Proceduren bergeben werden }


FUNCTION  HexB(b:Byte):String;     { Byte -> 2Byte Hex }
FUNCTION  HexW(w:Word):String;     { Word -> 4Byte Hex }
FUNCTION  HexL(l:LongInt):String;  { LongInt -> 8Byte Hex }
FUNCTION  HexP(p:Pointer):String;  { Pointer -> 4:4Byte Hex }
FUNCTION  BinB(b:Byte):String;     { Byte -> 8Bit Binr }
FUNCTION  BinW(w:Word):String;     { Word -> 16Bit Binr }
FUNCTION  BinL(l:LongInt):String;  { LongInt -> 32Bit Binr }

{ Zahlen <-> String mit Len Bytes, evtl. Auffllung mit Vornullen
  tritt bei Wandlung String -> Zahl ein Fehler auf, so Zahl=0 }
FUNCTION  IntToStr(l:LongInt;len:Byte):String;
FUNCTION  StrToInt(St:String):LongInt;
FUNCTION  RealToStr(r:Real; len,Dez:Byte):String;
FUNCTION  StrToReal(St:String):Real;

{ Entfernen von Leerzeichen links/rechts/an beiden Seiten}
FUNCTION  TrimLeft(St:String):String;
FUNCTION  TrimRight(St:String):String;
FUNCTION  Trim(St:String):String;

FUNCTION  TimeToStr(Hour,Min,Sec : Word):String; { Zeit -> hh:mm:ss }
FUNCTION  DateToStr(Day,Month,Year:Word):String; { Datum -> dd.mm.yy }

FUNCTION  UpCase(CH:Char):Char;        { Kleinbuchstabe -> Gro, auch  }
FUNCTION  LoCase(CH:Char):Char;        { Grobuchstabe -> Klein, auch  }
FUNCTION  StUpCase(St:String):String;  { String-UpCase }


{ String aus len Zeichen ch }
FUNCTION  CharStr(CH:Char; len:Byte):String;

{ Fllt st mit ch auf bis zur Lnge Len }
FUNCTION  FillStr(St:String; CH:Char; len:Byte):String; 

{ String der Lnge len mit st in der Mitte }
FUNCTION  Center(St:String; len:Byte):String; 

{ ermittelt letztes Vorkommen von SubStr in St, vgl. Standard.Pos }
FUNCTION  LastPos(SubStr,St:String):Byte;

{ Hngt an FileNamen fn Eweiterung ext an, falls noch keine vorher }
FUNCTION  DefaultExtension(fn,ext:String):String;

{ Gibt Filenamen fn die Erweiterung ext }
FUNCTION  ForceExtension(fn,ext:String):String;

{ ermittelt die Anzahl der Worte im String st, die mit Sep getrennt sind }
FUNCTION  ItemCount(St:String;Sep:CharSet):Byte;

{ liest das num-te Wort aus String St, gltige Trennzeichen stehen in Sep }
FUNCTION  ExtractItem(Num:Byte;St:String;Sep:CharSet):String;


IMPLEMENTATION

TYPE
  HiLo = RECORD
    LoWord : Word;
    HiWord : Word;
  END;

FUNCTION  HexB(b:Byte):String;
CONST
  HexChar : ARRAY[0..15] OF Char = '0123456789ABCDEF';
BEGIN
  HexB:=HexChar[b SHR 4]+HexChar[b AND $0F];
END;

FUNCTION  HexW(w:Word):String;
BEGIN
  HexW:=HexB(Hi(w))+HexB(Lo(w));
END;

FUNCTION  HexL(l:LongInt):String;
VAR
  HL : HiLo ABSOLUTE l;
BEGIN
  HexL:=HexW(HL.HiWord)+HexW(HL.LoWord);
END;

FUNCTION  HexP(p:Pointer):String;
VAR
  HL : HiLo ABSOLUTE p;
BEGIN
  HexP:=HexW(HL.HiWord)+':'+HexW(HL.LoWord);
END;

FUNCTION  BinB(b:Byte):String;
VAR
  s : String;
  i : Word;
BEGIN
  s:='';
  FOR i:=7 DOWNTO 0 DO
    IF (b AND (1 SHL i))>0 THEN
      s:=s+'1'
    ELSE
      s:=s+'0';
  BinB:=s;
END;

FUNCTION  BinW(w:Word):String;
VAR
  s : String;
  i : Word;
BEGIN
  s:='';
  FOR i:=15 DOWNTO 0 DO
    IF (w AND (1 SHL i))>0 THEN
      s:=s+'1'
    ELSE
      s:=s+'0';
  BinW:=s;
END;

FUNCTION  BinL(l:LongInt):String;
VAR
  HL : HiLo ABSOLUTE l;
BEGIN
  BinL:=BinW(HL.HiWord)+BinW(HL.LoWord);
END;

FUNCTION  IntToStr(l:LongInt;len:Byte):String;
VAR s:String;
BEGIN
  Str(l,s);
  WHILE Length(s)<len DO
    s:='0'+s;
  IntToStr:=s;
END;

FUNCTION  StrToInt(St:String):LongInt;
VAR
  w  : LongInt;
  Err: Integer;
BEGIN
  Val(St,w,Err);
  StrToInt:=w;
END;

FUNCTION  RealToStr(r:Real; len,Dez:Byte):String;
VAR St:String;
BEGIN
  Str(r:len:Dez,St);
  RealToStr:=St;
END;

FUNCTION  StrToReal(St:String):Real;
VAR
  r   : Real;
  Err : Integer;
BEGIN
  Val(St,r,Err);
  StrToReal:=r;
END;

FUNCTION  TrimLeft(St:String):String;
BEGIN
  WHILE (Length(St)>0) AND (St[1]=SPC) DO
    Delete(St,1,1);
  TrimLeft:=St;
END;

FUNCTION  TrimRight(St:String):String;
BEGIN
  WHILE (Length(St)>0) AND (St[Length(St)]=SPC) DO
    Delete(St,Length(St),1);
  TrimRight:=St;
END;

FUNCTION  Trim(St:String):String;
BEGIN
  Trim:=TrimLeft(TrimRight(St));
END;

FUNCTION  TimeToStr(Hour,Min,Sec : Word):String;
BEGIN
  TimeToStr:=IntToStr(Hour,2)+':'+
             IntToStr(Min,2)+':'+
             IntToStr(Sec,2);
END;

FUNCTION  DateToStr(Day,Month,Year:Word):String;
BEGIN
  DateToStr:=IntToStr(Day,2)+'.'+
             IntToStr(Month,2)+'.'+
             IntToStr(Year MOD 100,2);
END;

FUNCTION  UpCase(CH:Char):Char;
BEGIN
  CASE CH OF
    '' : UpCase:='';
    '' : UpCase:='';
    '' : UpCase:='';
    ELSE  UpCase:=System.UpCase(CH);
  END;
END;

FUNCTION  LoCase(CH:Char):Char;
BEGIN
  CASE CH OF
    '' : LoCase:='';
    '' : LoCase:='';
    '' : LoCase:='';
    ELSE BEGIN
      IF (CH>='A') AND (CH<='Z') THEN
        LoCase:=Chr(Ord(CH)+$20)
      ELSE
        LoCase:=CH;
    END;
  END;
END;

FUNCTION  StUpCase(St:String):String;
VAR
  i : Byte;
BEGIN
  FOR i:=1 TO Length(St) DO
    St[i]:=UpCase(St[i]);
  StUpCase:=St;
END;

FUNCTION  CharStr(CH:Char; len:Byte):String;
VAR
  St : String;
BEGIN
  St:='';
  WHILE Length(St)<len DO
    St:=St+CH;
  CharStr:=St;
END;

FUNCTION  FillStr(St:String; CH:Char; len:Byte):String;
BEGIN
  WHILE Length(St)<len DO
    St:=St+CH;
  FillStr:=Copy(St,1,len);
END;

FUNCTION  Center(St:String; len:Byte):String;
BEGIN
  WHILE Length(St)<len DO St:=' '+St+' ';
  Center:=Copy(St,1,len);
END;

FUNCTION  LastPos(SubStr: String; St: String): Byte;
VAR
  Found,len,Pos : Byte;
BEGIN
  Pos:=Length(St);
  len:=Length(SubStr);
  Found:=0;
  WHILE (Pos>0) AND (Found=0) DO BEGIN
    IF Copy(St,Pos,len)=SubStr THEN
      Found:=Pos;
    Dec(Pos);
  END;
  LastPos:=Found;
END;

FUNCTION  DefaultExtension(fn,ext:String):String;
BEGIN
  IF LastPos('.',fn)=0 THEN
    DefaultExtension:=fn+'.'+ext
  ELSE
    DefaultExtension:=fn;
END;

FUNCTION  ForceExtension(fn,ext:String):String;
BEGIN
  IF LastPos('.',fn)=0 THEN
    ForceExtension:=fn+'.'+ext
  ELSE
    ForceExtension:=Copy(fn,1,LastPos('.',fn)-1)+'.'+ext;
END;

FUNCTION  ItemCount(St:String;Sep:CharSet):Byte;
VAR
  i,Count,len : Byte;
BEGIN
  Count:=0; i:=1;
  len:=Length(St);
  WHILE i<=len DO BEGIN
    WHILE (i<=len) AND (St[i] IN Sep) DO Inc(i);
    IF i<=len THEN Inc(Count);
    WHILE (i<=len) AND NOT(St[i] IN Sep) DO Inc(i);
  END;
  ItemCount:=Count;
END;

FUNCTION  ExtractItem(Num:Byte;St:String;Sep:CharSet):String;
VAR
  i, Count, len : Byte;
  Extract       : String;
BEGIN
  Count:=0; i:=1;
  len:=Length(St);
  Extract:='';
  WHILE (i<=len) AND (Count<>Num) DO BEGIN
    WHILE (i<=len) AND (St[i] IN Sep) DO Inc(i);
    IF i<=len THEN Inc(Count);
    WHILE (i<=len) AND NOT(St[i] IN Sep) DO BEGIN
      IF Count=Num THEN
        Extract:=Extract+St[i];
      Inc(i);
    END;
  END;
  ExtractItem:=Extract;
END;

END. { UNIT STRINGS }
